@@ -29,7 +29,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* curl stuff */
 #include <curl/curl.h>
 
 struct Memory {
@@ -116,7 +115,6 @@ static int server_push_callback(CURL *parent,
   return CURL_PUSH_OK;
 }
 
-
 /*
  * Download a file over HTTP/2, take care of server push.
  */
@@ -127,9 +125,9 @@ int main(void)
   int transfers = 1; /* we start with one */
   int i;
 
-  CURLcode res = curl_global_init(CURL_GLOBAL_ALL);
-  if(res)
-    return (int)res;
+  CURLcode result = curl_global_init(CURL_GLOBAL_ALL);
+  if(result)
+    return (int)result;
 
   /* init a multi stack */
   multi = curl_multi_init();
@@ -151,12 +149,12 @@ int main(void)
     int still_running; /* keep number of running handles */
     int rc;
 
-    CURLMcode mcode = curl_multi_perform(multi, &still_running);
-    if(mcode)
+    CURLMcode mresult = curl_multi_perform(multi, &still_running);
+    if(mresult)
       break;
 
-    mcode = curl_multi_wait(multi, NULL, 0, 1000, &rc);
-    if(mcode)
+    mresult = curl_multi_wait(multi, NULL, 0, 1000, &rc);
+    if(mresult)
       break;
 
     /*
