@@ -319,12 +319,12 @@ typedef enum {
 #define ARG_FILE 3 /* requires an argument, usually a filename */
 
 #define ARG_TYPEMASK 0x03
-#define ARGTYPE(x) ((x) & ARG_TYPEMASK)
+#define ARGTYPE(x)   ((x) & ARG_TYPEMASK)
 
-#define ARG_DEPR 0x10 /* deprecated option */
+#define ARG_DEPR  0x10 /* deprecated option */
 #define ARG_CLEAR 0x20 /* clear cmdline argument */
-#define ARG_TLS 0x40 /* requires TLS support */
-#define ARG_NO 0x80 /* set if the option is documented as --no-* */
+#define ARG_TLS   0x40 /* requires TLS support */
+#define ARG_NO    0x80 /* set if the option is documented as --no-* */
 
 struct LongShort {
   const char *lname;  /* long name option */
@@ -372,24 +372,24 @@ ParameterError getparameter(const char *flag, const char *nextarg,
                             int max_recursive);
 
 #ifdef UNITTESTS
-void parse_cert_parameter(const char *cert_parameter,
-                          char **certname,
-                          char **passphrase);
+ParameterError parse_cert_parameter(const char *cert_parameter,
+                                    char **certname,
+                                    char **passphrase);
 #endif
 
 ParameterError parse_args(int argc, argv_item_t argv[]);
 
-#if defined(UNICODE) && defined(_WIN32) && !defined(UNDER_CE)
+#if defined(UNICODE) && defined(_WIN32)
 
-#define convert_UTF8_to_tchar(ptr) curlx_convert_UTF8_to_wchar((ptr))
-#define convert_tchar_to_UTF8(ptr) curlx_convert_wchar_to_UTF8((ptr))
-#define unicodefree(ptr) curlx_unicodefree(ptr)
+#define convert_UTF8_to_tchar(ptr) curlx_convert_UTF8_to_wchar(ptr)
+#define convert_tchar_to_UTF8(ptr) curlx_convert_wchar_to_UTF8(ptr)
+#define unicodefree(ptr)           curlx_free(ptr)
 
 #else
 
 #define convert_UTF8_to_tchar(ptr) (const char *)(ptr)
 #define convert_tchar_to_UTF8(ptr) (const char *)(ptr)
-#define unicodefree(ptr) do {} while(0)
+#define unicodefree(ptr)           do {} while(0)
 
 #endif
 
