@@ -34,7 +34,7 @@ import sys
 from statistics import mean
 from typing import Dict, Any, Optional, List
 
-from testenv import Env, Httpd, CurlClient, Caddy, ExecResult, NghttpxQuic, RunProfile
+from testenv import Env, Httpd, CurlClient, Caddy, ExecResult, NghttpxQuic, RunProfile, Dante
 
 log = logging.getLogger(__name__)
 
@@ -61,7 +61,14 @@ class Card:
 
     @classmethod
     def fmt_mbs(cls, val):
-        return f'{val/(1024*1024):0.000f} MB/s' if val >= 0 else '--'
+        if val is None or val < 0:
+            return '--'
+        if val >= (1024*1024):
+            return f'{val/(1024*1024):0.000f} MB/s'
+        elif val >= 1024:
+            return f'{val / 1024:0.000f} KB/s'
+        else:
+            return f'{val:0.000f} B/s'
 
     @classmethod
     def fmt_reqs(cls, val):
@@ -119,6 +126,8 @@ class Card:
             print(f'Version: {score["meta"]["curl_V"]}')
         if 'curl_features' in score["meta"]:
             print(f'Features: {score["meta"]["curl_features"]}')
+        if 'limit-rate' in score['meta']:
+            print(f'--limit-rate: {score["meta"]["limit-rate"]}')
         print(f'Samples Size: {score["meta"]["samples"]}')
         if 'handshakes' in score:
             print(f'{"Handshakes":<24} {"ipv4":25} {"ipv6":28}')
@@ -185,7 +194,12 @@ class ScoreRunner:
                  verbose: int,
                  curl_verbose: int,
                  download_parallel: int = 0,
-                 server_addr: Optional[str] = None):
+                 upload_parallel: int = 0,
+                 server_addr: Optional[str] = None,
+                 with_flame: bool = False,
+                 socks_args: Optional[List[str]] = None,
+                 limit_rate: Optional[str] = None,
+                 suppress_cl: bool = False):
         self.verbose = verbose
         self.env = env
         self.protocol = protocol
@@ -194,12 +208,23 @@ class ScoreRunner:
         self.server_port = server_port
         self._silent_curl = not curl_verbose
         self._download_parallel = download_parallel
+        self._upload_parallel = upload_parallel
+        self._with_flame = with_flame
+        self._socks_args = socks_args
+        self._limit_rate = limit_rate
+        self.suppress_cl = suppress_cl
 
     def info(self, msg):
         if self.verbose > 0:
             sys.stderr.write(msg)
             sys.stderr.flush()
 
+    def mk_curl_client(self):
+        return CurlClient(env=self.env, silent=self._silent_curl,
+                          server_addr=self.server_addr,
+                          with_flame=self._with_flame,
+                          socks_args=self._socks_args)
+
     def handshakes(self) -> Dict[str, Any]:
         props = {}
         sample_size = 5
@@ -215,8 +240,7 @@ class ScoreRunner:
                 hs_samples = []
                 errors = []
                 for _ in range(sample_size):
-                    curl = CurlClient(env=self.env, silent=self._silent_curl,
-                                      server_addr=self.server_addr)
+                    curl = self.mk_curl_client()
                     args = [
                         '--http3-only' if self.protocol == 'h3' else '--http2',
                         f'--{ipv}', f'https://{authority}/'
@@ -274,11 +298,11 @@ class ScoreRunner:
         profiles = []
         self.info('single...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_download(urls=[url], alpn_proto=self.protocol,
                                    no_save=True, with_headers=False,
-                                   with_profile=True)
+                                   with_profile=True,
+                                   limit_rate=self._limit_rate)
             err = self._check_downloads(r, count)
             if err:
                 errors.append(err)
@@ -295,11 +319,12 @@ class ScoreRunner:
         url = f'{url}?[0-{count - 1}]'
         self.info('serial...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_download(urls=[url], alpn_proto=self.protocol,
                                    no_save=True,
-                                   with_headers=False, with_profile=True)
+                                   with_headers=False,
+                                   with_profile=True,
+                                   limit_rate=self._limit_rate)
             err = self._check_downloads(r, count)
             if err:
                 errors.append(err)
@@ -317,12 +342,12 @@ class ScoreRunner:
         url = f'{url}?[0-{count - 1}]'
         self.info('parallel...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_download(urls=[url], alpn_proto=self.protocol,
                                    no_save=True,
                                    with_headers=False,
                                    with_profile=True,
+                                   limit_rate=self._limit_rate,
                                    extra_args=[
                                        '--parallel',
                                        '--parallel-max', str(max_parallel)
@@ -339,9 +364,12 @@ class ScoreRunner:
     def downloads(self, count: int, fsizes: List[int], meta: Dict[str, Any]) -> Dict[str, Any]:
         nsamples = meta['samples']
         max_parallel = self._download_parallel if self._download_parallel > 0 else count
-        cols = ['size', 'single']
+        cols = ['size']
+        if not self._download_parallel:
+            cols.append('single')
+            if count > 1:
+                cols.append(f'serial({count})')
         if count > 1:
-            cols.append(f'serial({count})')
             cols.append(f'parallel({count}x{max_parallel})')
         rows = []
         for fsize in fsizes:
@@ -351,16 +379,20 @@ class ScoreRunner:
             }]
             self.info(f'{row[0]["sval"]} downloads...')
             url = f'https://{self.env.domain1}:{self.server_port}/score{row[0]["sval"]}.data'
-
-            row.append(self.dl_single(url=url, nsamples=nsamples))
+            if 'single' in cols:
+                row.append(self.dl_single(url=url, nsamples=nsamples))
             if count > 1:
-                row.append(self.dl_serial(url=url, count=count, nsamples=nsamples))
+                if 'single' in cols:
+                    row.append(self.dl_serial(url=url, count=count, nsamples=nsamples))
                 row.append(self.dl_parallel(url=url, count=count, nsamples=nsamples))
             rows.append(row)
             self.info('done.\n')
+        title = f'Downloads from {meta["server"]}'
+        if self._socks_args:
+            title += f' via {self._socks_args}'
         return {
             'meta': {
-                'title': f'Downloads from {meta["server"]}',
+                'title': title,
                 'count': count,
                 'max-parallel': max_parallel,
             },
@@ -387,10 +419,10 @@ class ScoreRunner:
         profiles = []
         self.info('single...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_put(urls=[url], fdata=fpath, alpn_proto=self.protocol,
-                              with_headers=False, with_profile=True)
+                              with_headers=False, with_profile=True,
+                              suppress_cl=self.suppress_cl)
             err = self._check_uploads(r, 1)
             if err:
                 errors.append(err)
@@ -407,10 +439,10 @@ class ScoreRunner:
         url = f'{url}?id=[0-{count - 1}]'
         self.info('serial...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_put(urls=[url], fdata=fpath, alpn_proto=self.protocol,
-                              with_headers=False, with_profile=True)
+                              with_headers=False, with_profile=True,
+                              suppress_cl=self.suppress_cl)
             err = self._check_uploads(r, count)
             if err:
                 errors.append(err)
@@ -428,13 +460,13 @@ class ScoreRunner:
         url = f'{url}?id=[0-{count - 1}]'
         self.info('parallel...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_put(urls=[url], fdata=fpath, alpn_proto=self.protocol,
                               with_headers=False, with_profile=True,
+                              suppress_cl=self.suppress_cl,
                               extra_args=[
                                    '--parallel',
-                                   '--parallel-max', str(max_parallel)
+                                   '--parallel-max', str(max_parallel),
                               ])
             err = self._check_uploads(r, count)
             if err:
@@ -447,11 +479,16 @@ class ScoreRunner:
 
     def uploads(self, count: int, fsizes: List[int], meta: Dict[str, Any]) -> Dict[str, Any]:
         nsamples = meta['samples']
-        max_parallel = self._download_parallel if self._download_parallel > 0 else count
-        url = f'https://{self.env.domain2}:{self.server_port}/curltest/put'
-        cols = ['size', 'single']
-        if count > 1:
+        max_parallel = self._upload_parallel if self._upload_parallel > 0 else count
+        cols = ['size']
+        run_single = not self._upload_parallel
+        run_serial = not self._upload_parallel and count > 1
+        run_parallel = self._upload_parallel or count > 1
+        if run_single:
+            cols.append('single')
+        if run_serial:
             cols.append(f'serial({count})')
+        if run_parallel:
             cols.append(f'parallel({count}x{max_parallel})')
         rows = []
         for fsize in fsizes:
@@ -459,20 +496,25 @@ class ScoreRunner:
                 'val': fsize,
                 'sval': Card.fmt_size(fsize)
             }]
+            self.info(f'{row[0]["sval"]} uploads...')
+            url = f'https://{self.env.domain1}:{self.server_port}/curltest/put'
             fname = f'upload{row[0]["sval"]}.data'
             fpath = self._make_docs_file(docs_dir=self.env.gen_dir,
                                          fname=fname, fsize=fsize)
-
-            self.info(f'{row[0]["sval"]} uploads...')
-            row.append(self.ul_single(url=url, fpath=fpath, nsamples=nsamples))
-            if count > 1:
+            if run_single:
+                row.append(self.ul_single(url=url, fpath=fpath, nsamples=nsamples))
+            if run_serial:
                 row.append(self.ul_serial(url=url, fpath=fpath, count=count, nsamples=nsamples))
+            if run_parallel:
                 row.append(self.ul_parallel(url=url, fpath=fpath, count=count, nsamples=nsamples))
             rows.append(row)
             self.info('done.\n')
+        title = f'Uploads to {meta["server"]}'
+        if self._socks_args:
+            title += f' via {self._socks_args}'
         return {
             'meta': {
-                'title': f'Uploads to {meta["server"]}',
+                'title': title,
                 'count': count,
                 'max-parallel': max_parallel,
             },
@@ -494,8 +536,7 @@ class ScoreRunner:
             ])
         self.info(f'{max_parallel}...')
         for _ in range(nsamples):
-            curl = CurlClient(env=self.env, silent=self._silent_curl,
-                              server_addr=self.server_addr)
+            curl = self.mk_curl_client()
             r = curl.http_download(urls=[url], alpn_proto=self.protocol, no_save=True,
                                    with_headers=False, with_profile=True,
                                    with_stats=False, extra_args=extra_args)
@@ -517,7 +558,8 @@ class ScoreRunner:
         fsize = 10*1024
         cols = ['size', 'total']
         rows = []
-        cols.extend([f'{mp} max' for mp in [1, 6, 25, 50, 100, 300]])
+        mparallel = meta['request_parallels']
+        cols.extend([f'{mp} max' for mp in mparallel])
         row = [{
             'val': fsize,
             'sval': Card.fmt_size(fsize)
@@ -528,12 +570,15 @@ class ScoreRunner:
         self.info('requests, max parallel...')
         row.extend([self.do_requests(url=url, count=count,
                                      max_parallel=mp, nsamples=meta["samples"])
-                    for mp in [1, 6, 25, 50, 100, 300]])
+                    for mp in mparallel])
         rows.append(row)
         self.info('done.\n')
+        title = f'Requests in parallel to {meta["server"]}'
+        if self._socks_args:
+            title += f' via {self._socks_args}'
         return {
             'meta': {
-                'title': f'Requests in parallel to {meta["server"]}',
+                'title': title,
                 'count': count,
             },
             'cols': cols,
@@ -547,6 +592,7 @@ class ScoreRunner:
               uploads: Optional[List[int]] = None,
               upload_count: int = 50,
               req_count=5000,
+              request_parallels=None,
               nsamples: int = 1,
               requests: bool = True):
         self.info(f"scoring {self.protocol} against {self.server_descr}\n")
@@ -562,11 +608,14 @@ class ScoreRunner:
                 'date': f'{datetime.datetime.now(tz=datetime.timezone.utc).isoformat()}',
             }
         }
+        if self._limit_rate:
+            score['meta']['limit-rate'] = self._limit_rate
+
         if self.protocol == 'h3':
             score['meta']['protocol'] = 'h3'
             if not self.env.have_h3_curl():
                 raise ScoreCardError('curl does not support HTTP/3')
-            for lib in ['ngtcp2', 'quiche', 'msh3', 'nghttp3']:
+            for lib in ['ngtcp2', 'quiche', 'nghttp3']:
                 if self.env.curl_uses_lib(lib):
                     score['meta']['implementation'] = lib
                     break
@@ -599,6 +648,9 @@ class ScoreRunner:
                                             fsizes=uploads,
                                             meta=score['meta'])
         if requests:
+            if request_parallels is None:
+                request_parallels = [1, 6, 25, 50, 100, 300]
+            score['meta']['request_parallels'] = request_parallels
             score['requests'] = self.requests(count=req_count, meta=score['meta'])
         return score
 
@@ -624,6 +676,13 @@ def run_score(args, protocol):
             uploads.extend([Card.parse_size(s) for s in x.split(',')])
 
     requests = True
+    request_parallels = None
+    if args.request_parallels:
+        request_parallels = []
+        for x in args.request_parallels:
+            request_parallels.extend([int(s) for s in x.split(',')])
+
+
     if args.downloads or args.uploads or args.requests or args.handshakes:
         handshakes = args.handshakes
         if not args.downloads:
@@ -642,6 +701,20 @@ def run_score(args, protocol):
     env = Env()
     env.setup()
     env.test_timeout = None
+
+    sockd = None
+    socks_args = None
+    if args.socks4 and args.socks5:
+        raise ScoreCardError('unable to run --socks4 and --socks5 together')
+    elif args.socks4 or args.socks5:
+        sockd = Dante(env=env)
+    if sockd:
+        assert sockd.initial_start()
+        socks_args = [
+            '--socks4' if args.socks4 else '--socks5',
+            f'127.0.0.1:{sockd.port}',
+        ]
+
     httpd = None
     nghttpx = None
     caddy = None
@@ -663,7 +736,12 @@ def run_score(args, protocol):
                                server_port=remote_port,
                                verbose=args.verbose,
                                curl_verbose=args.curl_verbose,
-                               download_parallel=args.download_parallel)
+                               download_parallel=args.download_parallel,
+                               upload_parallel=args.upload_parallel,
+                               with_flame=args.flame,
+                               socks_args=socks_args,
+                               limit_rate=args.limit_rate,
+                               suppress_cl=args.upload_no_cl)
             cards.append(card)
 
         if test_httpd:
@@ -687,7 +765,11 @@ def run_score(args, protocol):
                                server_descr=server_descr,
                                server_port=server_port,
                                verbose=args.verbose, curl_verbose=args.curl_verbose,
-                               download_parallel=args.download_parallel)
+                               download_parallel=args.download_parallel,
+                               upload_parallel=args.upload_parallel,
+                               with_flame=args.flame,
+                               socks_args=socks_args,
+                               limit_rate=args.limit_rate)
             card.setup_resources(server_docs, downloads)
             cards.append(card)
 
@@ -711,7 +793,11 @@ def run_score(args, protocol):
                                server_descr=server_descr,
                                server_port=server_port,
                                verbose=args.verbose, curl_verbose=args.curl_verbose,
-                               download_parallel=args.download_parallel)
+                               download_parallel=args.download_parallel,
+                               upload_parallel=args.upload_parallel,
+                               with_flame=args.flame,
+                               socks_args=socks_args,
+                               limit_rate=args.limit_rate)
             card.setup_resources(server_docs, downloads)
             cards.append(card)
 
@@ -731,6 +817,7 @@ def run_score(args, protocol):
                                    upload_count=args.upload_count,
                                    req_count=args.request_count,
                                    requests=requests,
+                                   request_parallels=request_parallels,
                                    nsamples=args.samples)
                 if args.json:
                     print(json.JSONEncoder(indent=2).encode(score))
@@ -750,6 +837,8 @@ def run_score(args, protocol):
             nghttpx.stop(wait_dead=False)
         if httpd:
             httpd.stop()
+        if sockd:
+            sockd.stop()
     return rv
 
 
@@ -765,41 +854,15 @@ def print_file(filename):
 
 def main():
     parser = argparse.ArgumentParser(prog='scorecard', description="""
-        Run a range of tests to give a scorecard for a HTTP protocol
+        Run a range of tests to give a scorecard for an HTTP protocol
         'h3' or 'h2' implementation in curl.
         """)
     parser.add_argument("-v", "--verbose", action='count', default=1,
                         help="log more output on stderr")
     parser.add_argument("-j", "--json", action='store_true',
                         default=False, help="print json instead of text")
-    parser.add_argument("-H", "--handshakes", action='store_true',
-                        default=False, help="evaluate handshakes only")
-    parser.add_argument("-d", "--downloads", action='store_true',
-                        default=False, help="evaluate downloads")
-    parser.add_argument("--download-sizes", action='append', type=str,
-                        metavar='numberlist',
-                        default=None, help="evaluate download size")
-    parser.add_argument("--download-count", action='store', type=int,
-                        metavar='number',
-                        default=50, help="perform that many downloads")
     parser.add_argument("--samples", action='store', type=int, metavar='number',
                         default=1, help="how many sample runs to make")
-    parser.add_argument("--download-parallel", action='store', type=int,
-                        metavar='number', default=0,
-                        help="perform that many downloads in parallel (default all)")
-    parser.add_argument("-u", "--uploads", action='store_true',
-                        default=False, help="evaluate uploads")
-    parser.add_argument("--upload-sizes", action='append', type=str,
-                        metavar='numberlist',
-                        default=None, help="evaluate upload size")
-    parser.add_argument("--upload-count", action='store', type=int,
-                        metavar='number', default=50,
-                        help="perform that many uploads")
-    parser.add_argument("-r", "--requests", action='store_true',
-                        default=False, help="evaluate requests")
-    parser.add_argument("--request-count", action='store', type=int,
-                        metavar='number',
-                        default=5000, help="perform that many requests")
     parser.add_argument("--httpd", action='store_true', default=False,
                         help="evaluate httpd server only")
     parser.add_argument("--caddy", action='store_true', default=False,
@@ -814,6 +877,52 @@ def main():
                         help="only start the servers")
     parser.add_argument("--remote", action='store', type=str,
                         default=None, help="score against the remote server at <ip>:<port>")
+    parser.add_argument("--flame", action='store_true',
+                        default = False, help="produce a flame graph on curl")
+    parser.add_argument("--limit-rate", action='store', type=str,
+                        default=None, help="use curl's --limit-rate")
+
+    parser.add_argument("-H", "--handshakes", action='store_true',
+                        default=False, help="evaluate handshakes only")
+
+    parser.add_argument("-d", "--downloads", action='store_true',
+                        default=False, help="evaluate downloads")
+    parser.add_argument("--download-sizes", action='append', type=str,
+                        metavar='numberlist',
+                        default=None, help="evaluate download size")
+    parser.add_argument("--download-count", action='store', type=int,
+                        metavar='number',
+                        default=50, help="perform that many downloads")
+    parser.add_argument("--download-parallel", action='store', type=int,
+                        metavar='number', default=0,
+                        help="perform that many downloads in parallel (default all)")
+
+    parser.add_argument("-u", "--uploads", action='store_true',
+                        default=False, help="evaluate uploads")
+    parser.add_argument("--upload-sizes", action='append', type=str,
+                        metavar='numberlist',
+                        default=None, help="evaluate upload size")
+    parser.add_argument("--upload-count", action='store', type=int,
+                        metavar='number', default=50,
+                        help="perform that many uploads")
+    parser.add_argument("--upload-parallel", action='store', type=int,
+                        metavar='number', default=0,
+                        help="perform that many uploads in parallel (default all)")
+    parser.add_argument("--upload-no-cl", action='store_true',
+                        default=False, help="suppress content-length on upload")
+
+    parser.add_argument("-r", "--requests", action='store_true',
+                        default=False, help="evaluate requests")
+    parser.add_argument("--request-count", action='store', type=int,
+                        metavar='number',
+                        default=5000, help="perform that many requests")
+    parser.add_argument("--request-parallels", action='append', type=str,
+                        metavar='numberlist',
+                        default=None, help="evaluate request with these max-parallel numbers")
+    parser.add_argument("--socks4", action='store_true',
+                        default=False, help="test with SOCKS4 proxy")
+    parser.add_argument("--socks5", action='store_true',
+                        default=False, help="test with SOCKS5 proxy")
     args = parser.parse_args()
 
     if args.verbose > 0:
