@@ -21,7 +21,7 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
+#include "first.h"
 
 /*
   Check range/resume returned error codes and data presence.
@@ -34,13 +34,11 @@
 
 */
 
-#include "memdebug.h"
-
-#define F_RESUME        (1 << 0)        /* resume/range. */
-#define F_HTTP416       (1 << 1)        /* Server returns http code 416. */
-#define F_FAIL          (1 << 2)        /* Fail on error. */
-#define F_CONTENTRANGE  (1 << 3)        /* Server sends content-range hdr. */
-#define F_IGNOREBODY    (1 << 4)        /* Body should be ignored. */
+#define F_RESUME       (1 << 0)        /* resume/range. */
+#define F_HTTP416      (1 << 1)        /* Server returns http code 416. */
+#define F_FAIL         (1 << 2)        /* Fail on error. */
+#define F_CONTENTRANGE (1 << 3)        /* Server sends content-range hdr. */
+#define F_IGNOREBODY   (1 << 4)        /* Body should be ignored. */
 
 struct testparams {
   unsigned int flags; /* ORed flags as above. */
@@ -71,12 +69,12 @@ static const struct testparams testparams[] = {
                                                                    CURLE_OK }
 };
 
-static int      hasbody;
+static int hasbody;
 
 static size_t writedata(char *data, size_t size, size_t nmemb, void *userdata)
 {
-  (void) data;
-  (void) userdata;
+  (void)data;
+  (void)userdata;
 
   if(size && nmemb)
     hasbody = 1;
@@ -86,39 +84,39 @@ static size_t writedata(char *data, size_t size, size_t nmemb, void *userdata)
 static int onetest(CURL *curl, const char *url, const struct testparams *p,
                    size_t num)
 {
-  CURLcode res;
+  CURLcode result;
   unsigned int replyselector;
   char urlbuf[256];
 
   replyselector = (p->flags & F_CONTENTRANGE) ? 1 : 0;
   if(p->flags & F_HTTP416)
     replyselector += 2;
-  msnprintf(urlbuf, sizeof(urlbuf), "%s%04u", url, replyselector);
+  curl_msnprintf(urlbuf, sizeof(urlbuf), "%s%04u", url, replyselector);
   test_setopt(curl, CURLOPT_URL, urlbuf);
   test_setopt(curl, CURLOPT_VERBOSE, 1L);
   test_setopt(curl, CURLOPT_RESUME_FROM, (p->flags & F_RESUME) ? 3L : 0L);
   test_setopt(curl, CURLOPT_RANGE, !(p->flags & F_RESUME) ?
-                                   "3-1000000": (char *) NULL);
+                                   "3-1000000" : (char *)NULL);
   test_setopt(curl, CURLOPT_FAILONERROR, (p->flags & F_FAIL) ? 1L : 0L);
   hasbody = 0;
-  res = curl_easy_perform(curl);
-  if(res != p->result) {
-    printf("%zd: bad error code (%d): resume=%s, fail=%s, http416=%s, "
-           "content-range=%s, expected=%d\n", num, res,
-           (p->flags & F_RESUME) ? "yes": "no",
-           (p->flags & F_FAIL) ? "yes": "no",
-           (p->flags & F_HTTP416) ? "yes": "no",
-           (p->flags & F_CONTENTRANGE) ? "yes": "no",
-           p->result);
+  result = curl_easy_perform(curl);
+  if(result != p->result) {
+    curl_mprintf("%zu: bad error code (%d): resume=%s, fail=%s, http416=%s, "
+                 "content-range=%s, expected=%d\n", num, result,
+                 (p->flags & F_RESUME) ? "yes" : "no",
+                 (p->flags & F_FAIL) ? "yes" : "no",
+                 (p->flags & F_HTTP416) ? "yes" : "no",
+                 (p->flags & F_CONTENTRANGE) ? "yes" : "no",
+                 p->result);
     return 1;
   }
   if(hasbody && (p->flags & F_IGNOREBODY)) {
-    printf("body should be ignored and is not: resume=%s, fail=%s, "
-           "http416=%s, content-range=%s\n",
-           (p->flags & F_RESUME) ? "yes": "no",
-           (p->flags & F_FAIL) ? "yes": "no",
-           (p->flags & F_HTTP416) ? "yes": "no",
-           (p->flags & F_CONTENTRANGE) ? "yes": "no");
+    curl_mprintf("body should be ignored and is not: resume=%s, fail=%s, "
+                 "http416=%s, content-range=%s\n",
+                 (p->flags & F_RESUME) ? "yes" : "no",
+                 (p->flags & F_FAIL) ? "yes" : "no",
+                 (p->flags & F_HTTP416) ? "yes" : "no",
+                 (p->flags & F_CONTENTRANGE) ? "yes" : "no");
     return 1;
   }
   return 0;
@@ -131,22 +129,22 @@ test_cleanup:
 /* for debugging: */
 /* #define SINGLETEST 9 */
 
-CURLcode test(char *URL)
+static CURLcode test_lib1156(const char *URL)
 {
-  CURLcode res;
+  CURLcode result;
   CURL *curl;
   size_t i;
   int status = 0;
 
   if(curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
-    fprintf(stderr, "curl_global_init() failed\n");
+    curl_mfprintf(stderr, "curl_global_init() failed\n");
     return TEST_ERR_MAJOR_BAD;
   }
 
   for(i = 0; i < CURL_ARRAYSIZE(testparams); i++) {
     curl = curl_easy_init();
     if(!curl) {
-      fprintf(stderr, "curl_easy_init() failed\n");
+      curl_mfprintf(stderr, "curl_easy_init() failed\n");
       curl_global_cleanup();
       return TEST_ERR_MAJOR_BAD;
     }
@@ -161,7 +159,7 @@ CURLcode test(char *URL)
   }
 
   curl_global_cleanup();
-  printf("%d\n", status);
+  curl_mprintf("%d\n", status);
   return (CURLcode)status;
 
 test_cleanup:
@@ -169,5 +167,5 @@ test_cleanup:
   curl_easy_cleanup(curl);
   curl_global_cleanup();
 
-  return res;
+  return result;
 }
