@@ -21,17 +21,15 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
-
-#include "memdebug.h"
+#include "first.h"
 
 /* Test CURLINFO_RESPONSE_CODE */
 
-int test(char *URL)
+static CURLcode test_lib1532(const char *URL)
 {
   CURL *curl;
   long httpcode;
-  CURLcode res = CURLE_OK;
+  CURLcode result = CURLE_OK;
 
   global_init(CURL_GLOBAL_ALL);
 
@@ -39,44 +37,49 @@ int test(char *URL)
 
   easy_setopt(curl, CURLOPT_URL, URL);
 
-  res = curl_easy_perform(curl);
-  if(res) {
-    fprintf(stderr, "%s:%d curl_easy_perform() failed with code %d (%s)\n",
-            __FILE__, __LINE__, res, curl_easy_strerror(res));
+  result = curl_easy_perform(curl);
+  if(result) {
+    curl_mfprintf(stderr,
+                  "%s:%d curl_easy_perform() failed with code %d (%s)\n",
+                  __FILE__, __LINE__, result, curl_easy_strerror(result));
     goto test_cleanup;
   }
 
-  res = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpcode);
-  if(res) {
-    fprintf(stderr, "%s:%d curl_easy_getinfo() failed with code %d (%s)\n",
-            __FILE__, __LINE__, res, curl_easy_strerror(res));
+  result = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpcode);
+  if(result) {
+    curl_mfprintf(stderr,
+                  "%s:%d curl_easy_getinfo() failed with code %d (%s)\n",
+                  __FILE__, __LINE__, result, curl_easy_strerror(result));
     goto test_cleanup;
   }
   if(httpcode != 200) {
-    fprintf(stderr, "%s:%d unexpected response code %ld\n",
-            __FILE__, __LINE__, httpcode);
-    res = CURLE_HTTP_RETURNED_ERROR;
+    curl_mfprintf(stderr, "%s:%d unexpected response code %ld\n",
+                  __FILE__, __LINE__, httpcode);
+    result = CURLE_HTTP_RETURNED_ERROR;
     goto test_cleanup;
   }
 
   /* Test for a regression of github bug 1017 (response code does not reset) */
   curl_easy_reset(curl);
 
-  res = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpcode);
-  if(res) {
-    fprintf(stderr, "%s:%d curl_easy_getinfo() failed with code %d (%s)\n",
-            __FILE__, __LINE__, res, curl_easy_strerror(res));
+  result = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpcode);
+  if(result) {
+    curl_mfprintf(stderr,
+                  "%s:%d curl_easy_getinfo() failed with code %d (%s)\n",
+                  __FILE__, __LINE__, result, curl_easy_strerror(result));
     goto test_cleanup;
   }
   if(httpcode) {
-    fprintf(stderr, "%s:%d curl_easy_reset failed to zero the response code\n"
-            "possible regression of github bug 1017\n", __FILE__, __LINE__);
-    res = CURLE_HTTP_RETURNED_ERROR;
+    curl_mfprintf(stderr,
+                  "%s:%d curl_easy_reset failed to zero the response code\n"
+                  "possible regression of github bug 1017\n",
+                  __FILE__, __LINE__);
+    result = CURLE_HTTP_RETURNED_ERROR;
     goto test_cleanup;
   }
 
 test_cleanup:
   curl_easy_cleanup(curl);
   curl_global_cleanup();
-  return (int)res;
+  return result;
 }
