@@ -23,8 +23,6 @@
  ***************************************************************************/
 #include "first.h"
 
-#include "memdebug.h"
-
 struct headerinfo {
   size_t largest;
 };
@@ -46,8 +44,8 @@ static CURLcode test_lib1556(const char *URL)
 {
   CURLcode code;
   CURL *curl = NULL;
-  CURLcode res = CURLE_OK;
-  struct headerinfo info = {0};
+  CURLcode result = CURLE_OK;
+  struct headerinfo info = { 0 };
 
   global_init(CURL_GLOBAL_ALL);
 
@@ -63,7 +61,7 @@ static CURLcode test_lib1556(const char *URL)
     curl_mfprintf(stderr, "%s:%d curl_easy_perform() failed, "
                   "with code %d (%s)\n",
                   __FILE__, __LINE__, code, curl_easy_strerror(code));
-    res = TEST_ERR_MAJOR_BAD;
+    result = TEST_ERR_MAJOR_BAD;
     goto test_cleanup;
   }
 
@@ -74,5 +72,5 @@ test_cleanup:
   curl_easy_cleanup(curl);
   curl_global_cleanup();
 
-  return res;
+  return result;
 }
