@@ -24,12 +24,11 @@
 #include "first.h"
 
 #include "testtrace.h"
-#include "memdebug.h"
 
 static CURLcode test_lib2502(const char *URL)
 {
-  CURLcode res = CURLE_OK;
-  CURL *curl[NUM_HANDLES] = {0};
+  CURLcode result = CURLE_OK;
+  CURL *curl[NUM_HANDLES] = { 0 };
   int running;
   CURLM *multi = NULL;
   size_t i;
@@ -41,8 +40,7 @@ static CURLcode test_lib2502(const char *URL)
 
   (void)URL;
 
-  curl_msnprintf(dnsentry, sizeof(dnsentry), "localhost:%s:%s",
-                 port, address);
+  curl_msnprintf(dnsentry, sizeof(dnsentry), "localhost:%s:%s", port, address);
   curl_mprintf("%s\n", dnsentry);
   slist = curl_slist_append(slist, dnsentry);
   if(!slist) {
@@ -64,8 +62,7 @@ static CURLcode test_lib2502(const char *URL)
     easy_init(curl[i]);
     /* specify target */
     curl_msnprintf(target_url, sizeof(target_url),
-                   "https://localhost:%s/path/2502%04zu",
-                   port, i + 1);
+                   "https://localhost:%s/path/2502%04zu", port, i + 1);
     target_url[sizeof(target_url) - 1] = '\0';
     easy_setopt(curl[i], CURLOPT_URL, target_url);
     /* go http2 */
@@ -136,5 +133,5 @@ test_cleanup:
   curl_multi_cleanup(multi);
   curl_global_cleanup();
 
-  return res;
+  return result;
 }
