@@ -26,118 +26,82 @@ package sshhelp;
 
 use strict;
 use warnings;
-use Exporter;
+
+BEGIN {
+    use base qw(Exporter);
+
+    our @EXPORT_OK = qw(
+        $sshdexe
+        $sshexe
+        $sftpsrvexe
+        $sftpexe
+        $sshkeygenexe
+        $sshdconfig
+        $sshconfig
+        $sftpconfig
+        $knownhosts
+        $sshdlog
+        $sshlog
+        $sftplog
+        $sftpcmds
+        $hstprvkeyf
+        $hstpubkeyf
+        $hstpubmd5f
+        $hstpubsha256f
+        $cliprvkeyf
+        $clipubkeyf
+        display_file_top
+        display_sshdconfig
+        display_sshconfig
+        display_sftpconfig
+        display_sshdlog
+        display_sshlog
+        display_sftplog
+        dump_array
+        find_sshd
+        find_ssh
+        find_sftpsrv
+        find_sftp
+        find_sshkeygen
+        find_httptlssrv
+        sshversioninfo
+    );
+}
+
 use File::Spec;
 
-
-#***************************************************************************
-# Global symbols allowed without explicit package name
-#
-use vars qw(
-    @ISA
-    @EXPORT_OK
-    $sshdexe
-    $sshexe
-    $sftpsrvexe
-    $sftpexe
-    $sshkeygenexe
-    $httptlssrvexe
-    $sshdconfig
-    $sshconfig
-    $sftpconfig
-    $knownhosts
-    $sshdlog
-    $sshlog
-    $sftplog
-    $sftpcmds
-    $hstprvkeyf
-    $hstpubkeyf
-    $hstpubmd5f
-    $hstpubsha256f
-    $cliprvkeyf
-    $clipubkeyf
-    @sftppath
-    @httptlssrvpath
-    );
-
-
-#***************************************************************************
-# Inherit Exporter's capabilities
-#
-@ISA = qw(Exporter);
-
-
-#***************************************************************************
-# Global symbols this module will export upon request
-#
-@EXPORT_OK = qw(
-    $sshdexe
-    $sshexe
-    $sftpsrvexe
-    $sftpexe
-    $sshkeygenexe
-    $sshdconfig
-    $sshconfig
-    $sftpconfig
-    $knownhosts
-    $sshdlog
-    $sshlog
-    $sftplog
-    $sftpcmds
-    $hstprvkeyf
-    $hstpubkeyf
-    $hstpubmd5f
-    $hstpubsha256f
-    $cliprvkeyf
-    $clipubkeyf
-    display_sshdconfig
-    display_sshconfig
-    display_sftpconfig
-    display_sshdlog
-    display_sshlog
-    display_sftplog
-    dump_array
+use pathhelp qw(
     exe_ext
-    find_sshd
-    find_ssh
-    find_sftpsrv
-    find_sftp
-    find_sshkeygen
-    find_httptlssrv
-    logmsg
-    sshversioninfo
     );
 
-
 #***************************************************************************
 # Global variables initialization
 #
-$sshdexe         = 'sshd'        .exe_ext('SSH'); # base name and ext of ssh daemon
-$sshexe          = 'ssh'         .exe_ext('SSH'); # base name and ext of ssh client
-$sftpsrvexe      = 'sftp-server' .exe_ext('SSH'); # base name and ext of sftp-server
-$sftpexe         = 'sftp'        .exe_ext('SSH'); # base name and ext of sftp client
-$sshkeygenexe    = 'ssh-keygen'  .exe_ext('SSH'); # base name and ext of ssh-keygen
-$httptlssrvexe   = 'gnutls-serv' .exe_ext('SSH'); # base name and ext of gnutls-serv
-$sshdconfig      = 'curl_sshd_config';       # ssh daemon config file
-$sshconfig       = 'curl_ssh_config';        # ssh client config file
-$sftpconfig      = 'curl_sftp_config';       # sftp client config file
-$sshdlog         = undef;                    # ssh daemon log file
-$sshlog          = undef;                    # ssh client log file
-$sftplog         = undef;                    # sftp client log file
-$sftpcmds        = 'curl_sftp_cmds';         # sftp client commands batch file
-$knownhosts      = 'curl_client_knownhosts'; # ssh knownhosts file
-$hstprvkeyf      = 'curl_host_rsa_key';      # host private key file
-$hstpubkeyf      = 'curl_host_rsa_key.pub';  # host public key file
-$hstpubmd5f      = 'curl_host_rsa_key.pub_md5';  # md5 hash of host public key
-$hstpubsha256f   = 'curl_host_rsa_key.pub_sha256';  # sha256 hash of host public key
-$cliprvkeyf      = 'curl_client_key';        # client private key file
-$clipubkeyf      = 'curl_client_key.pub';    # client public key file
-
+our $sshdexe         = 'sshd'        .exe_ext('SSH'); # base name and ext of ssh daemon
+our $sshexe          = 'ssh'         .exe_ext('SSH'); # base name and ext of ssh client
+our $sftpsrvexe      = 'sftp-server' .exe_ext('SSH'); # base name and ext of sftp-server
+our $sftpexe         = 'sftp'        .exe_ext('SSH'); # base name and ext of sftp client
+our $sshkeygenexe    = 'ssh-keygen'  .exe_ext('SSH'); # base name and ext of ssh-keygen
+our $httptlssrvexe   = 'gnutls-serv' .exe_ext('SSH'); # base name and ext of gnutls-serv
+our $sshdconfig      = 'curl_sshd_config';       # ssh daemon config file
+our $sshconfig       = 'curl_ssh_config';        # ssh client config file
+our $sftpconfig      = 'curl_sftp_config';       # sftp client config file
+our $sshdlog         = undef;                    # ssh daemon log file
+our $sshlog          = undef;                    # ssh client log file
+our $sftplog         = undef;                    # sftp client log file
+our $sftpcmds        = 'curl_sftp_cmds';         # sftp client commands batch file
+our $knownhosts      = 'curl_client_knownhosts'; # ssh knownhosts file
+our $hstprvkeyf      = 'curl_host_rsa_key';      # host private key file
+our $hstpubkeyf      = 'curl_host_rsa_key.pub';  # host public key file
+our $hstpubmd5f      = 'curl_host_rsa_key.pub_md5';  # md5 hash of host public key
+our $hstpubsha256f   = 'curl_host_rsa_key.pub_sha256';  # sha256 hash of host public key
+our $cliprvkeyf      = 'curl_client_key';        # client private key file
+our $clipubkeyf      = 'curl_client_key.pub';    # client public key file
 
 #***************************************************************************
 # Absolute paths where to look for sftp-server plugin, when not in PATH
 #
-@sftppath = qw(
+our @sftppath = qw(
     /usr/lib/openssh
     /usr/libexec/openssh
     /usr/libexec
@@ -159,11 +123,10 @@ $clipubkeyf      = 'curl_client_key.pub';    # client public key file
     /opt/ssh/libexec
     );
 
-
 #***************************************************************************
 # Absolute paths where to look for httptlssrv (gnutls-serv), when not in PATH
 #
-@httptlssrvpath = qw(
+our @httptlssrvpath = qw(
     /usr/sbin
     /usr/libexec
     /usr/lib
@@ -183,25 +146,6 @@ $clipubkeyf      = 'curl_client_key.pub';    # client public key file
     /opt/gnutls/libexec
     );
 
-
-#***************************************************************************
-# Return file extension for executable files on this operating system
-#
-sub exe_ext {
-    my ($component, @arr) = @_;
-    if ($ENV{'CURL_TEST_EXE_EXT'}) {
-        return $ENV{'CURL_TEST_EXE_EXT'};
-    }
-    if ($ENV{'CURL_TEST_EXE_EXT_'.$component}) {
-        return $ENV{'CURL_TEST_EXE_EXT_'.$component};
-    }
-    if ($^O eq 'MSWin32' || $^O eq 'cygwin' || $^O eq 'msys' ||
-        $^O eq 'dos' || $^O eq 'os2') {
-        return '.exe';
-    }
-}
-
-
 #***************************************************************************
 # Create or overwrite the given file with lines from an array of strings
 #
@@ -212,12 +156,12 @@ sub dump_array {
     if(!$filename) {
         $error = 'Error: Missing argument 1 for dump_array()';
     }
-    elsif(open(TEXTFH, ">$filename")) {
+    elsif(open(my $textfh, ">", $filename)) {
         foreach my $line (@arr) {
-            $line .= "\n" unless($line =~ /\n$/);
-            print TEXTFH $line;
+            $line .= "\n" if($line !~ /\n$/);
+            print $textfh $line;
         }
-        if(!close(TEXTFH)) {
+        if(!close($textfh)) {
             $error = "Error: cannot close file $filename";
         }
     }
@@ -227,33 +171,34 @@ sub dump_array {
     return $error;
 }
 
-
-#***************************************************************************
-# Display a message
-#
-sub logmsg {
-    my ($line) = @_;
-    chomp $line if($line);
-    $line .= "\n";
-    print "$line";
-}
-
-
 #***************************************************************************
 # Display contents of the given file
 #
 sub display_file {
     my $filename = $_[0];
     print "=== Start of file $filename\n";
-    if(open(DISPLAYFH, "<$filename")) {
-        while(my $line = <DISPLAYFH>) {
+    if(open(my $displayfh, "<", "$filename")) {
+        while(my $line = <$displayfh>) {
             print "$line";
         }
-        close DISPLAYFH;
+        close $displayfh;
     }
     print "=== End of file $filename\n";
 }
 
+#***************************************************************************
+# Display first line of the given file
+#
+sub display_file_top {
+    my $filename = $_[0];
+    print "=== Top of file $filename\n";
+    if(open(my $displayfh, "<", "$filename")) {
+        my $line = <$displayfh>;
+        print "$line";
+        close $displayfh;
+    }
+    print "=== End of file $filename\n";
+}
 
 #***************************************************************************
 # Display contents of the ssh daemon config file
@@ -262,7 +207,6 @@ sub display_sshdconfig {
     display_file($sshdconfig);
 }
 
-
 #***************************************************************************
 # Display contents of the ssh client config file
 #
@@ -270,7 +214,6 @@ sub display_sshconfig {
     display_file($sshconfig);
 }
 
-
 #***************************************************************************
 # Display contents of the sftp client config file
 #
@@ -278,7 +221,6 @@ sub display_sftpconfig {
     display_file($sftpconfig);
 }
 
-
 #***************************************************************************
 # Display contents of the ssh daemon log file
 #
@@ -287,7 +229,6 @@ sub display_sshdlog {
     display_file($sshdlog);
 }
 
-
 #***************************************************************************
 # Display contents of the ssh client log file
 #
@@ -296,7 +237,6 @@ sub display_sshlog {
     display_file($sshlog);
 }
 
-
 #***************************************************************************
 # Display contents of the sftp client log file
 #
@@ -305,7 +245,6 @@ sub display_sftplog {
     display_file($sftplog);
 }
 
-
 #***************************************************************************
 # Find a file somewhere in the given path
 #
@@ -319,9 +258,9 @@ sub find_file {
             return $file;
         }
     }
+    return "";
 }
 
-
 #***************************************************************************
 # Find an executable file somewhere in the given path
 #
@@ -337,9 +276,9 @@ sub find_exe_file {
             return $file if(($xext) && (lc($file) =~ /\Q$xext\E$/));
         }
     }
+    return "";
 }
 
-
 #***************************************************************************
 # Find a file in environment path or in our sftppath
 #
@@ -351,7 +290,6 @@ sub find_file_spath {
     return find_file($filename, @spath);
 }
 
-
 #***************************************************************************
 # Find an executable file in environment path or in our httptlssrvpath
 #
@@ -363,7 +301,6 @@ sub find_exe_file_hpath {
     return find_exe_file($filename, @hpath);
 }
 
-
 #***************************************************************************
 # Find ssh daemon and return canonical filename
 #
@@ -371,7 +308,6 @@ sub find_sshd {
     return find_file_spath($sshdexe);
 }
 
-
 #***************************************************************************
 # Find ssh client and return canonical filename
 #
@@ -379,7 +315,6 @@ sub find_ssh {
     return find_file_spath($sshexe);
 }
 
-
 #***************************************************************************
 # Find sftp-server plugin and return canonical filename
 #
@@ -387,7 +322,6 @@ sub find_sftpsrv {
     return find_file_spath($sftpsrvexe);
 }
 
-
 #***************************************************************************
 # Find sftp client and return canonical filename
 #
@@ -395,7 +329,6 @@ sub find_sftp {
     return find_file_spath($sftpexe);
 }
 
-
 #***************************************************************************
 # Find ssh-keygen and return canonical filename
 #
@@ -403,24 +336,25 @@ sub find_sshkeygen {
     return find_file_spath($sshkeygenexe);
 }
 
-
 #***************************************************************************
 # Find httptlssrv (gnutls-serv) and return canonical filename
 #
 sub find_httptlssrv {
     my $p = find_exe_file_hpath($httptlssrvexe);
-    my @o = `$p -l`;
-    my $found;
-    for(@o) {
-        if(/Key exchange: SRP/) {
-            $found = 1;
-            last;
+    if($p) {
+        my @o = `"$p" -l`;
+        my $found;
+        for(@o) {
+            if(/Key exchange: SRP/) {
+                $found = 1;
+                last;
+            }
         }
+        return $p if($found);
     }
-    return $p if($found);
+    return "";
 }
 
-
 #***************************************************************************
 # Return version info for the given ssh client or server binaries
 #
@@ -481,7 +415,6 @@ sub sshversioninfo {
     return ($sshid, $versnum, $versstr, $error);
 }
 
-
 #***************************************************************************
 # End of library
 1;
